#include "Hooks.h"
#include "UnicodeString.h"
#include "ProcessName.h"

NtUserGetClipboardData_T g_pNtUserGetClipboardData = NULL;
NtUserSetClipboardData_T g_pNtUserSetClipboardData = NULL;

//////////////////////////////////////////////////////////////////////////

typedef struct _CLIPBOARD_CONTEXT
{
    UNICODE_STRING usFrom;
    UNICODE_STRING usTo;
} 
CLIPBOARD_CONTEXT, *PCLIPBOARD_CONTEXT;

static PCLIPBOARD_CONTEXT g_pContext = NULL;

//////////////////////////////////////////////////////////////////////////

VOID FreeContext(PCLIPBOARD_CONTEXT pContext)
{
    if (pContext)
    {
        RtlFreeUnicodeString(&pContext->usFrom);
        RtlFreeUnicodeString(&pContext->usTo);

        ExFreePool(pContext);
    }
}

NTSTATUS CreateContext(PCLIPBOARD_CONTEXT* ppContext)
{
    FreeContext(*ppContext);

    *ppContext = ExAllocatePool(PagedPool, sizeof(CLIPBOARD_CONTEXT));
    if (*ppContext == NULL)
        return STATUS_INSUFFICIENT_RESOURCES;

    RtlZeroMemory(*ppContext, sizeof(CLIPBOARD_CONTEXT));

    return STATUS_SUCCESS;
}

//////////////////////////////////////////////////////////////////////////

BOOLEAN IsPasteAllowed(PCLIPBOARD_CONTEXT pContext)
{
    UNICODE_STRING usWordpad, usNotepad;

    RtlInitUnicodeString(&usWordpad, L"wordpad.exe");
    RtlInitUnicodeString(&usNotepad, L"notepad.exe");

    if ((RtlCompareUnicodeString(&pContext->usFrom, &usWordpad, TRUE) == 0) &&
        (RtlCompareUnicodeString(&pContext->usTo, &usNotepad, TRUE) == 0))
    {
        DbgPrint("Copying from WORDPAD.EXE to NOTEPAD.EXE is denied\n");
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

//////////////////////////////////////////////////////////////////////////

ULONG __stdcall NtUserSetClipboardData_Hook(ULONG uFormat, HANDLE hParam, PULONG pParam)
{
    ULONG uRetVal;
    NTSTATUS status;

    uRetVal = g_pNtUserSetClipboardData(uFormat, hParam, pParam);

    // Create clipboard context to store process names
    status = CreateContext(&g_pContext);
    if (!NT_SUCCESS(status))
        goto cleanup;

    // Save name of the process, which puts data to the clipboard
    status = AllocUnicodeString(&g_pContext->usFrom, 256);
    if (!NT_SUCCESS(status))
        goto cleanup;

    status = GetCurrentProcessName(&g_pContext->usFrom);
    if (!NT_SUCCESS(status))
        goto cleanup;

cleanup:
    if (!NT_SUCCESS(status))
        FreeContext(g_pContext);

    return uRetVal;
}

//////////////////////////////////////////////////////////////////////////

HANDLE __stdcall NtUserGetClipboardData_Hook(ULONG uFormat, PVOID pParam)
{
    HANDLE hRetVal;
    NTSTATUS status;

    // Call the original function
    hRetVal = g_pNtUserGetClipboardData(uFormat, pParam);

    if (g_pContext == NULL)
        goto cleanup;

    // Save name of the process, which gets data from the clipboard
    status = AllocUnicodeString(&g_pContext->usTo, 256);
    if (!NT_SUCCESS(status))
        goto cleanup;

    status = GetCurrentProcessName(&g_pContext->usTo);
    if (!NT_SUCCESS(status))
        goto cleanup;

    // If paste is denied, return NULL to the caller
    if (!IsPasteAllowed(g_pContext))
        hRetVal = NULL;

cleanup:
    return hRetVal;
}
