#include "SetHooks.h"
#include "Hooks.h"

#include "GetProcAddr.h"
#include "PatchFunction.h"

#include "ProcessEnum.h"
#include "Process.h"
#include "ModuleInfo.h"

//////////////////////////////////////////////////////////////////////////

static MASK_BUFFER g_bufNtUserSetClipboardData = {
    "\x50\x50\xFF\x75\x0C\xFF\x75\x08\xE8\x00\x00\x00\x00\x8B\xF0",
    "\xFE\xFE\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00\x00\x00\x00\xFF\xFF",
    15
};

static MASK_BUFFER g_bufNtUserGetClipboardData = {
    "\x8B\x45\x08\x89\x45\x00\x8D\x4D\x00\x51\x50\x50\xE8",
    "\xFF\xFF\xFF\xFF\xFF\x00\xFF\xFF\x00\xFF\xFF\xF0\xFF",
    13
};

static MASK_BUFFER g_bufNtUserGetClipboardDataStart = {
    "\x6A\x00\x68",
    "\xFF\x00\xFF",
    3
};

static MASK_BUFFER g_bufNtUserSetClipboardDataStart = {
    "\x6A\x00\x68",
    "\xFF\x00\xFF",
    3
};

//////////////////////////////////////////////////////////////////////////

static NTSTATUS PatchClipboardFunctions(BUFFER* pWin32k)
{
    void* pNtUserGetClipboardData = NULL;
    void* pNtUserSetClipboardData = NULL;
    PLSTATUS result = PL_STATUS_SUCCESS;

    // Find NtUserGetClipboardData
    result = PlGetProcAddress(pWin32k, 
        &g_bufNtUserGetClipboardData, 
        180, 
        &g_bufNtUserGetClipboardDataStart, 
        &pNtUserGetClipboardData);
    if (result != PL_STATUS_SUCCESS)
        goto end;

    // Find NtUserSetClipboardData
    result = PlGetProcAddress(pWin32k, 
        &g_bufNtUserSetClipboardData, 
        180, 
        &g_bufNtUserSetClipboardDataStart, 
        &pNtUserSetClipboardData);
    if (result != PL_STATUS_SUCCESS)
        goto end;

    // Patch 'em all
    KdPrint(("NtUserGetClipboardData=0x%X NtUserSetClipboardData=0x%X\n", 
        pNtUserGetClipboardData, 
        pNtUserSetClipboardData));

    g_pNtUserGetClipboardData = PatchFunction(pNtUserGetClipboardData, NtUserGetClipboardData_Hook);
    if (g_pNtUserGetClipboardData == NULL)
        return STATUS_INSUFFICIENT_RESOURCES;

    g_pNtUserSetClipboardData = PatchFunction(pNtUserSetClipboardData, NtUserSetClipboardData_Hook);
    if (g_pNtUserSetClipboardData == NULL)
        return STATUS_INSUFFICIENT_RESOURCES;

end:
    if (result == PL_STATUS_SUCCESS)
        return STATUS_SUCCESS;

    if (result == PL_STATUS_NOT_FOUND)
        return STATUS_NOT_FOUND;

    return STATUS_UNSUCCESSFUL;
}

static NTSTATUS PatchWin32kSys(PEPROCESS pProcess, BUFFER* pWin32k)
{
    KAPC_STATE apcState;
    NTSTATUS status = STATUS_SUCCESS;

    KeStackAttachProcess(pProcess, &apcState);
    KdPrint(("Patching win32k.sys... Attached to process 0x%X\n", pProcess));

    // If win32k.sys is mapped, patch Clipboard functions
    if (MmIsAddressValid((PVOID)pWin32k->pStart))
    {
        status = PatchClipboardFunctions(pWin32k);

        // If everything's OK, end the enumeration
        if (NT_SUCCESS(status))
            status = 1;
    }

    KeUnstackDetachProcess(&apcState);
    return status;
}

static NTSTATUS Callback(SYSTEM_PROCESS_INFORMATION * pProcessInfo)
{
    SYSTEM_MODULE modWin32k;

    // We need information about win32k.sys
    NTSTATUS status = GetModuleInfo("win32k.sys", &modWin32k);
    if (!NT_SUCCESS(status))
        return status;

    if (pProcessInfo->UniqueProcessId != 0) // If not the Idle process
    {
        PEPROCESS pProcess = 0;
        BUFFER bufWin32k = {modWin32k.pAddress, modWin32k.dSize};

        // Get EPROCESS pointer
        status = PsLookupProcessByProcessId(pProcessInfo->UniqueProcessId, &pProcess);
        if (!NT_SUCCESS(status))
            return status;

        // Try to patch
        status = PatchWin32kSys(pProcess, &bufWin32k);
        
        // Now, release EPROCESS
        ObDereferenceObject(pProcess);
    }

    return status;
}

NTSTATUS SetHooks()
{
    return ProcessEnum(Callback);
}