#include "ProcessName.h"
#include "SystemInfo.h"

NTSTATUS GetCurrentProcessName(PUNICODE_STRING pProcessName)
{
    ULONG ulSysInfoSize = 0;
    PSYSTEM_PROCESS_INFORMATION pSysInfo = 0;
    PSYSTEM_PROCESS_INFORMATION pInfoEntry = 0;
    NTSTATUS status = STATUS_SUCCESS;
    HANDLE hProcessId = 0;

    ASSERT(KeGetCurrentIrql() == PASSIVE_LEVEL);

    // Get information about all processes in the system
    status = SystemInfoQuery(&pSysInfo, SystemProcessInformation, &ulSysInfoSize);
    if (!NT_SUCCESS(status))
        return status;

    // Get current process ID
    hProcessId = PsGetCurrentProcessId();

    // Lookup through the process list
    pInfoEntry = pSysInfo;
    do
    {
        if (pInfoEntry->UniqueProcessId == hProcessId)
        {
            // Great! We've found our process
            RtlCopyUnicodeString(pProcessName, &pInfoEntry->ImageName);
            
            status = STATUS_SUCCESS;
            goto cleanup;
        }
        pInfoEntry = SpiNextEntry(pInfoEntry);
    } 
    while (pInfoEntry);

    // Sad. Something gone wrong
    status = STATUS_NOT_FOUND;

cleanup:
    if (pSysInfo)
        SystemInfoFree(pSysInfo);

    return status;
}