#include "SystemInfo.h"

NTSTATUS SystemInfoQuery(PVOID* retBuf, SYSTEMINFOCLASS infoClass, PULONG resSize)
{
    ULONG dwSize = 0x400;
    NTSTATUS status;

    // Try to get information until buffer will be big enough
    *retBuf = NULL;
    do 
    {
        // Free buffer, if allocated
        if (*retBuf)
            ExFreePool(*retBuf);

        // Increase size
        dwSize *= 2;

        // Allocate new buffer
        *retBuf = ExAllocatePool(PagedPool, dwSize);
        if(*retBuf == 0)
            return STATUS_INSUFFICIENT_RESOURCES;

        // Hope the buffer will have enough size
        status = ZwQuerySystemInformation(infoClass, *retBuf, dwSize, resSize);

    } while (status == STATUS_INFO_LENGTH_MISMATCH);

    if (!NT_SUCCESS(status)) // Something else has gone wrong
    {
        ExFreePool(*retBuf);
        *retBuf = NULL;
    }

    return status;
}

//////////////////////////////////////////////////////////////////////////

NTSTATUS SystemInfoFree(PVOID sysInfoBuf)
{
    if (sysInfoBuf)
        ExFreePool(sysInfoBuf);

    return STATUS_SUCCESS;
}

PSYSTEM_PROCESS_INFORMATION SpiNextEntry(PSYSTEM_PROCESS_INFORMATION pEntry)
{
    if (!pEntry->NextEntryOffset)
        return NULL;

   return (PSYSTEM_PROCESS_INFORMATION)((PCHAR)pEntry + pEntry->NextEntryOffset);
}