#include "CodePatch.h"
#include "hde32.h"

#include <ntifs.h>

size_t CodeWriteCall(code_t* pFrom, const code_t* pTo)
{
    const code_t cInstrCode = '\xE8';                   // CALL
    const size_t nInstSize = 1 + sizeof(void*);

    *(code_t*)(pFrom + 0) = cInstrCode;                 // Instruction code
    *(size_t*)(pFrom + 1) = pTo - (pFrom + nInstSize);  // Offset

    return nInstSize;
}

size_t CodeWriteJump(code_t* pFrom, const code_t* pTo)
{
    const code_t cInstrCode = '\xE9';                   // JUMP
    const size_t nInstSize = 1 + sizeof(void*);

    *(code_t*)(pFrom + 0) = cInstrCode;                 // Instruction code
    *(size_t*)(pFrom + 1) = pTo - (pFrom + nInstSize);  // Offset

    return nInstSize;
}

void CodeFreeBuffer(code_t* pCode, size_t nSize)
{
    RtlFillMemory(pCode, nSize, 0x90);
}

code_t* CodeAllocate(size_t nSize)
{
    code_t* pCode = (code_t*)ExAllocatePool(PagedPool, nSize);
    if (pCode == NULL)
        return NULL;

    CodeFreeBuffer(pCode, nSize);

    return pCode;
}

size_t CodeDisplaceInstruction(code_t* pNew, code_t* pOld)
{
    // Parse instruction
    hde32s hs;
    hde32_disasm(pOld, &hs);

    // Copy instruction
    RtlMoveMemory(pNew, pOld, hs.len);

    // Correct relative address, if there's any
    if ((hs.flags & F_REL8) == F_REL8)
        *(uint8_t* )(pNew + hs.imm_offset) -= (pNew - pOld);

    if ((hs.flags & F_REL16) == F_REL16)
        *(uint16_t*)(pNew + hs.imm_offset) -= (pNew - pOld);

    if ((hs.flags & F_REL32) == F_REL32)
        *(uint32_t*)(pNew + hs.imm_offset) -= (pNew - pOld);

    return hs.len;
}

size_t CodeDisplace(code_t* pDestination, code_t* pCode, size_t nSize)
{
    size_t nMoved = 0;

    while (nMoved < nSize)
        nMoved += CodeDisplaceInstruction(pDestination + nMoved, pCode + nMoved);
    CodeFreeBuffer(pCode, nMoved);

    return nMoved;
}

size_t CodeJumpSize()
{
    return 5;
}