#include "GetProcAddr.h"
#include "MemSearch.h"

int PlSingatureDuplicates(const BUFFER* pModule, 
                          const MASK_BUFFER* pSignature, 
                          const char* pMatchedSignature)
{
    const char* pTemp=0;
    BUFFER bufTemp = {pMatchedSignature + 1, pModule->nSize - (int)(pMatchedSignature - pModule->pStart)};
    return (PlSearchSequence(pSignature, &bufTemp, &pTemp) == PL_STATUS_SUCCESS);
}

//////////////////////////////////////////////////////////////////////////

size_t PlCalculateSearchAreaSize(const BUFFER* pModule, const char* pSignature, size_t nMaxSize)
{
    size_t nSignatureOffset = (size_t)(pSignature - pModule->pStart);
    
    return (nSignatureOffset < nMaxSize) ? nSignatureOffset : nMaxSize;
}

//////////////////////////////////////////////////////////////////////////

static PLSTATUS PlGetProcStart(const BUFFER* pModule, 
                               const char* pSequenceStart, 
                               size_t nMaxSignOffset, 
                               const MASK_BUFFER* pFncStart,
                               void ** ppAddress)
{
    // Define search area
    size_t nSignatureOffset = (size_t)(pSequenceStart - pModule->pStart);
    size_t nSearchArea = PlCalculateSearchAreaSize(pModule, pSequenceStart, nMaxSignOffset);

    BUFFER bufFncSearchArea = {pSequenceStart, nSearchArea};

    // Search back for the function start bytes
    return PlSearchSequenceBack(pFncStart, &bufFncSearchArea, (const char**)ppAddress);
}

PLSTATUS PlGetProcAddress(const BUFFER* pModule, 
                          const MASK_BUFFER* pSignature, 
                          size_t nMaxSignOffset,
                          const MASK_BUFFER* pFncStart, 
                          void ** ppAddress)
{
    const char* pSequenceStart=0;

    // check unique key
    int status = PlSearchSequence(pSignature, pModule, &pSequenceStart);
    if(status != PL_STATUS_SUCCESS)
        return status;

    // check duplicate
    if (PlSingatureDuplicates(pModule, pSignature, pSequenceStart))
        return PL_STATUS_SIGNATURE_NOT_UNIQUE;

    // now find the start of the function
    return PlGetProcStart(pModule, pSequenceStart, nMaxSignOffset, pFncStart, ppAddress);
}
