#include "PatchFunction.h"
#include "CodePatch.h"

#include <ntifs.h>

//////////////////////////////////////////////////////////////////////////

ULONG DisableWriteProtection()
{
    ULONG OldCr0=0;

    __asm
    {
        cli
        mov     eax, cr0
        mov     OldCr0, eax     // save old CR0
        and     eax, 0xFFFEFFFF // reset WP bit
        mov     cr0, eax
    }
    return OldCr0;
}

VOID EnableWriteProtection(ULONG OldCr0)
{
    __asm
    {
        mov     eax, OldCr0
        mov     cr0, eax        // restore old CR0
        sti
    }
}

//////////////////////////////////////////////////////////////////////////

void* PatchFunction(void* pFunction, void* pHook)
{
    code_t* pFncStart = NULL;
    size_t nPosition = 0;

    ULONG oldCR0 = DisableWriteProtection();

    // Move some first bytes of original function 
    pFncStart = CodeAllocate(100);
    if (pFncStart == NULL)
        return NULL;

    nPosition = CodeDisplace(pFncStart, pFunction, CodeJumpSize());
    nPosition += CodeWriteJump(pFncStart + nPosition, (code_t*)pFunction + nPosition);

    // Patch original function
    CodeWriteJump(pFunction, pHook);

    EnableWriteProtection(oldCR0);
    return pFncStart;
}
